@@ -115,12 +115,16 @@ void uobject_root_init(sObject* self)
     uobject_init(readline);
     uobject_put(self, "rl", readline);
 
+    uobject_put(readline, "file_name_completion_null_generator", NFUN_NEW_GC(cmd_readline_file_name_completion_null_generator, NULL, TRUE));
+    uobject_put(readline, "forced_update_display", NFUN_NEW_GC(cmd_readline_forced_update_display, NULL, TRUE));
     uobject_put(readline, "insert_text", NFUN_NEW_GC(cmd_readline_insert_text, NULL, TRUE));
     uobject_put(readline, "delete_text", NFUN_NEW_GC(cmd_readline_delete_text, NULL, TRUE));
     uobject_put(readline, "clear_screen", NFUN_NEW_GC(cmd_readline_clear_screen, NULL, TRUE));
     uobject_put(readline, "point_move", NFUN_NEW_GC(cmd_readline_point_move, NULL, TRUE));
     uobject_put(readline, "read_history", NFUN_NEW_GC(cmd_readline_read_history, NULL, TRUE));
     uobject_put(readline, "write_history", NFUN_NEW_GC(cmd_readline_write_history, NULL, TRUE));
+    uobject_put(readline, "replace_line", NFUN_NEW_GC(cmd_readline_replace_line, NULL, TRUE));
+    uobject_put(readline, "point", NFUN_NEW_GC(cmd_readline_point, NULL, TRUE));
     uobject_put(self, "completion", NFUN_NEW_GC(cmd_completion, NULL, TRUE));
     uobject_put(self, "p", NFUN_NEW_GC(cmd_p, NULL, TRUE));
     uobject_put(self, "jobs", NFUN_NEW_GC(cmd_jobs, NULL, TRUE));
@@ -229,9 +233,6 @@ void uobject_root_init(sObject* self)
     uobject_put(self, "sort", NFUN_NEW_GC(cmd_sort, NULL, TRUE));
     uobject_put(self, "readline", NFUN_NEW_GC(cmd_readline, NULL, TRUE));
     uobject_put(self, "kanjicode", NFUN_NEW_GC(cmd_kanjicode, NULL, TRUE));
-#if defined(HAVE_MIGEMO_H)
-    uobject_put(self, "migemo_match", NFUN_NEW_GC(cmd_migemo_match, NULL, TRUE));
-#endif
 
     uobject_put(self, "sub", NFUN_NEW_GC(cmd_sub, NULL, TRUE));
     uobject_put(self, "time", NFUN_NEW_GC(cmd_time, NULL, TRUE));
