@@ -567,6 +567,10 @@ BOOL cmd_objinfo(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_kanjicode(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_defined(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_funinfo(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
+BOOL cmd_readline_forced_update_display(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
+BOOL cmd_readline_file_name_completion_null_generator(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
+BOOL cmd_readline_replace_line(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
+BOOL cmd_readline_point(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_readline_read_history(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_readline_write_history(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL cmd_readline_point_move(sObject* nextin, sObject* nextout, sRunInfo* runinfo);
@@ -729,13 +733,13 @@ BOOL gc_valid_object(sObject* object);
 
 BOOL parse(char* p, char* sname, int* sline, sObject* block, sObject** current_object);
 
-#define RCODE_BREAK 256
-#define RCODE_RETURN 257
-#define RCODE_EXIT 258
-#define RCODE_SIGNAL_INTERRUPT 259
-#define RCODE_NFUN_FALSE 260
-#define RCODE_NFUN_INVALID_USSING 261
-#define RCODE_NFUN_NULL_INPUT 262
+#define RCODE_BREAK 0x100
+#define RCODE_RETURN 0x200
+#define RCODE_EXIT 0x400
+#define RCODE_SIGNAL_INTERRUPT 0x800
+#define RCODE_NFUN_FALSE 0x1000
+#define RCODE_NFUN_INVALID_USSING 0x2000
+#define RCODE_NFUN_NULL_INPUT 0x4000
 //#define RCODE_COMMAND_NOT_FOUND 262
 
 void xyzsh_kill_all_jobs();
@@ -753,6 +757,7 @@ extern sRunInfo* gRunInfoOfRunningObject;
 BOOL bufsiz_write(int fd, char* buf, int buf_len);
 
 BOOL load_file(char* fname, sObject* nextin, sObject* nextout, sRunInfo* runinfo, char** argv, int argc);
+BOOL load_so_file(char* fname, sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 
 int object_gc_children_mark(sObject* self);
 
@@ -783,11 +788,6 @@ void xyzsh_read_rc_core(char* path);
 void run_init(enum eAppType app_type);
 void run_final();
 
-#if defined(HAVE_MIGEMO_H)
-void migemo_init();
-void migemo_final();
-#endif
-
 extern sObject* gPrompt;
 extern sObject* gDirStack;
 
@@ -815,6 +815,7 @@ BOOL xyzsh_readline_interface_onetime(int* rcode, char* cmdline, int cursor_poin
 char* xyzsh_job_title(int n);
 int xyzsh_job_num();
 BOOL xyzsh_run(int* rcode, sObject* block, char* source_name, fXyzshJobDone xyzsh_job_done_, sObject* nextin, sObject* nextout, int argc, char** argv, sObject* current_object);
+BOOL xyzsh_eval(int* rcode, char* cmd, char* source_name, fXyzshJobDone xyzsh_job_done_, sObject* nextin, sObject* nextout, int argc, char** argv, sObject* current_object);
 
 void err_msg(char* msg, char* sname, int line, char* command);
 void err_msg_adding(char* msg, char* sname, int line, char* command);
